/*****************************************************************************
 * Author:   Valient Gough <vgough@pobox.com>
 *
 *****************************************************************************
 * Copyright (c) 2002-2003, Valient Gough
 *
 * This library is free software; you can distribute it and/or modify it under
 * the terms of the GNU General Public License (GPL), as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GPL in the file COPYING for more
 * details.
 *
 */

#ifndef _Botan_incl_
#define _Botan_incl_

#include "Cipher.h"
#include "Interface.h"

#include <string>

namespace Botan
{
    class BlockCipher;
}

class BotanCipher : public Cipher
{
    rel::Interface _iface;
    int _keyLength;
    std::string _cipherName;
    std::string _macName;
    Botan::BlockCipher *blockCipher;
    int _macLength;

public:
    BotanCipher( const rel::Interface &iface, const char *cipherName, 
	    int keyLen );
    virtual ~BotanCipher();

    virtual rel::Interface interface() const;

    // create a new key based on a password
    virtual CipherKey newKey(const char *password, int passwdLength);
    // create a new random key
    virtual CipherKey newRandomKey();

    // data must be len keySize()
    virtual CipherKey readKey(const unsigned char *data, 
	                       const CipherKey &encodingKey);
    virtual void writeKey(const CipherKey &key, unsigned char *data, 
	                  const CipherKey &encodingKey); 
    virtual bool compareKey(const CipherKey &A, const CipherKey &B) const;

    // meta-data about the cypher
    virtual int keySize() const;
    virtual int encodedKeySize() const;
    virtual int cipherBlockSize() const;

    virtual uint64_t MAC_64( const unsigned char *src, int len,
	    const CipherKey &key ) const;

    // functional interfaces
    virtual bool streamEncode(unsigned char *buf, int len, 
	    uint64_t iv64, const CipherKey &key) const;
    virtual bool streamDecode(unsigned char *buf, int len, 
	    uint64_t iv64, const CipherKey &key) const;

    virtual bool blockEncode(unsigned char *buf, int size, 
	                     uint64_t iv64, const CipherKey &key) const;
    virtual bool blockDecode(unsigned char *buf, int size, 
	                     uint64_t iv64, const CipherKey &key) const;

private:
    bool streamCode(unsigned char *buf, int size,
	    uint64_t iv64, const CipherKey &key, bool enc, int rounds) const;
    bool blockCode(unsigned char *buf, int size,
	    uint64_t iv64, const CipherKey &key, bool enc) const;
};


#endif

